<?php

namespace Teknisa\Libs\Util\Repositories;

use Teknisa\Libs\Util\Utilities;

class Migration {
    const INSERT_LAST_MIG_QUERY = "INSERT INTO VERSAOOBJETO (NMOBJETOBD, NRVEROBJBD, NRORG, DTINCLUSAO, CDOPERINCLUSAO, NRORGINCLUSAO)
     VALUES (:NMOBJETOBD, :NRVEROBJBD, :NRORG, SYSDATE, :CDOPERADOR, :NRORG)";

    const UPDATE_LAST_MIG_QUERY = "UPDATE VERSAOOBJETO
   SET NRVEROBJBD   = :NRVEROBJBD,
       DTULTATU     = SYSDATE,
       CDOPERULTATU = :CDOPERADOR,
       NRORGULTATU  = :NRORG
 WHERE NMOBJETOBD = :NMOBJETOBD
   AND NRORG = :NRORG";
    /**
     * @param $productId
     * @param $customEntityManager
     * @return array
     */
    public static function getMigrationInfo($productId, $customEntityManager) {
        $query = "SELECT RPT.NAME, RPT.URL, RPT.PATH, PRP.ID,
                         PRP.DSDIRRELMIGBD AS MIG_RELATIVE_PATH,
                         DRP.INITIAL_MIGRATION_NUMBER
                    FROM OM_PRODUTO PDT
                   INNER JOIN OM_PRODUTO_REPOSITORIO PRP ON PRP.NRPRODUTO_ID = PDT.ID
                   INNER JOIN REPOSITORY RPT ON RPT.ID = PRP.NREXTERNAL_REPOSITORY_ID
                    LEFT JOIN DEPLOY_REPOSITORY DRP ON DRP.EXTERNAL_REPOSITORY_ID = PRP.ID
                   WHERE NREXTERNAL_PRODUCT_ID = :PRODUCT_ID
                     AND RPT.STATUS = 'A'
                     AND PRP.IDATIVO = 'S'";

        $entityManager = $customEntityManager ? $customEntityManager : Utilities::getEntityManager();
        $params = array( 'PRODUCT_ID' => $productId );
        return $entityManager->getConnection()->fetchAll($query, $params);
    }

    /**
     * @param $productId
     * @param $customEntityManager
     * @return array
     */
    public static function getMasterProductRepository($productId, $customEntityManager) {
        $query = "SELECT RPT.NAME, RPT.URL, RPT.PATH
                    FROM REPOSITORY RPT
                   WHERE PRODUCT = :PRODUCT_ID
                     AND RPT.STATUS = 'A'
                     AND RPT.IS_MASTER = 'Y'";

        $params = array( 'PRODUCT_ID' => $productId );
        $entityManager = $customEntityManager ? $customEntityManager : Utilities::getEntityManager();
        return $entityManager->getConnection()->fetchAll($query, $params);
    }

    /**
     * @param $nmObject
     * @param $nrOrg
     * @param $customEntityManager
     * @return array
     */
    public static function getLastAppliedMigration($nmObject, $nrOrg, $customEntityManager) {
        $query = "SELECT NMOBJETOBD, NRVEROBJBD, NRORG
                    FROM VERSAOOBJETO
                   WHERE NMOBJETOBD = :NMOBJETOBD
                     AND NRORG = :NRORG";

        $params = array( 'NMOBJETOBD' => $nmObject, 'NRORG' => $nrOrg );
        $entityManager = $customEntityManager ? $customEntityManager : Utilities::getEntityManager();
        return $entityManager->getConnection()->fetchAll($query, $params);
    }

    /**
     * @param $script
     * @param $customEntityManager
     * @return \Doctrine\DBAL\Driver\Statement
     * @throws \Doctrine\DBAL\DBALException
     */
    public static function executeScript($script, $customEntityManager) {
        $entityManager = $customEntityManager ? $customEntityManager : Utilities::getEntityManager();
        return $entityManager->getConnection()->executeQuery($script);
    }

    /**
     * @param $nmObject
     * @param $fileNumber
     * @param $nrOrg
     * @param $cdOperador
     * @param $customEntityManager
     * @return \Doctrine\DBAL\Driver\Statement
     * @throws \Doctrine\DBAL\DBALException
     */
    public static function insertLastMigration($nmObject, $fileNumber, $nrOrg, $cdOperador, $customEntityManager) {
        $params = array(
            'NMOBJETOBD' => $nmObject,
            'NRVEROBJBD' => $fileNumber,
            'NRORG'      => $nrOrg,
            'CDOPERADOR' => $cdOperador
        );
        $entityManager = $customEntityManager ? $customEntityManager : Utilities::getEntityManager();
        return $entityManager->getConnection()->executeQuery(self::INSERT_LAST_MIG_QUERY, $params);
    }

    /**
     * @param $nmObject
     * @param $fileNumber
     * @param $nrOrg
     * @param $cdOperador
     * @param $customEntityManager
     * @return \Doctrine\DBAL\Driver\Statement
     * @throws \Doctrine\DBAL\DBALException
     */
    public static function updateLastMigration($nmObject, $fileNumber, $nrOrg, $cdOperador, $customEntityManager) {
        $params = array(
            'NMOBJETOBD' => $nmObject,
            'NRVEROBJBD' => $fileNumber,
            'NRORG'      => $nrOrg,
            'CDOPERADOR' => $cdOperador
        );
        $entityManager = $customEntityManager ? $customEntityManager : Utilities::getEntityManager();
        return $entityManager->getConnection()->executeQuery(self::UPDATE_LAST_MIG_QUERY, $params);
    }
}